@@ -389,6 +389,7 @@ import re
 
 from abc import ABC, abstractmethod
 from copy import deepcopy
+from math import cos, sin, sqrt
 from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
 
 from ae.base import (                                                                                   # type: ignore
@@ -406,7 +407,7 @@ from ae.core import DEBUG_LEVELS, registered_app_names
 from ae.console import ConsoleApp                                                                       # type: ignore
 
 
-__version__ = '0.3.88'
+__version__ = '0.3.89'
 
 
 APP_STATE_SECTION_NAME = 'aeAppState'           #: config section name to store app state
@@ -461,6 +462,17 @@ if stack_var('__name__') == '__main__':
     check_all()                                                                                 # pragma: no cover
 
 
+def ellipse_polar_radius(ell_a: float, ell_b: float, radian: float) -> float:
+    """ calculate the radius from polar for the given ellipse and radian.
+
+    :param ell_a:               ellipse x-radius.
+    :param ell_b:               ellipse y-radius.
+    :param radian:              radian of angle.
+    :return:                    ellipse radius at the angle specified by :paramref:`.radian`.
+    """
+    return ell_a * ell_b / sqrt((ell_a * sin(radian)) ** 2 + (ell_b * cos(radian)) ** 2)
+
+
 def ensure_tap_kwargs_refs(init_kwargs: EventKwargsType, tap_widget: Any):
     """ ensure that the passed widget.__init__ kwargs dict contains a reference to itself within kwargs['tap_kwargs'].
 
@@ -872,14 +884,13 @@ class MainAppBase(ConsoleApp, ABC):
                                 flow path and the action `'focus'` will give the indexed widget the input focus (these
                                 exceptions are configurable via :data:`ACTIONS_CHANGING_FLOW_WITHOUT_CONFIRMATION`).
         """
-        msg = f" MainAppBase.change_flow({new_flow_id!r}, {event_kwargs}) id={self.flow_id!r} path={self.flow_path}"
-        self.vpo("CHECK   " + msg)
-
+        self.vpo(f"MainAppBase.change_flow({new_flow_id!r}, {event_kwargs}) id={self.flow_id!r} path={self.flow_path}")
+        prefix = " " * 12
         action = flow_action(new_flow_id)
         if not self.call_method(flow_change_confirmation_event_name(new_flow_id), flow_key(new_flow_id), event_kwargs) \
                 and not self.on_flow_change(new_flow_id, event_kwargs) \
                 and action not in ACTIONS_CHANGING_FLOW_WITHOUT_CONFIRMATION:
-            self.vpo("REJECTED" + msg)
+            self.vpo(f"{prefix}REJECTED")
             return False
 
         has_flow_focus = flow_action(self.flow_id) == 'focus'
@@ -887,7 +898,10 @@ class MainAppBase(ConsoleApp, ABC):
         if action in ACTIONS_EXTENDING_FLOW_PATH:
             if action == 'edit' and self.flow_path_action() == 'edit' \
                     and flow_key_split(self.flow_path[-1])[0] == flow_key_split(new_flow_id)[0]:
-                self.flow_path.pop()
+                _flow_id = self.flow_path.pop()
+                self.vpo(f"{prefix}PATH EDIT: removed '{_flow_id}' and adding {new_flow_id} to {self.flow_path}")
+            else:
+                self.vpo(f"{prefix}PATH EXTEND: appending '{new_flow_id}' to {self.flow_path}")
             self.flow_path.append(new_flow_id)
             self.change_app_state('flow_path', self.flow_path)
             flow_id = empty_flow if action == 'enter' else new_flow_id
@@ -895,9 +909,10 @@ class MainAppBase(ConsoleApp, ABC):
             # dismiss gets sent sometimes twice (e.g. on heavy double-clicking on drop-down-open-buttons)
             # .. therefore prevent run-time error
             if not self.flow_path:
-                self.dpo("IGNORED reduce on empty flow path" + msg)
+                self.vpo(f"{prefix}PATH REDUCE: cancelled flow change because of empty flow path")
                 return False
             ended_flow_id = self.flow_path.pop()
+            self.vpo(f"{prefix}PATH REDUCE: popped '{ended_flow_id}' from {self.flow_path}")
             self.change_app_state('flow_path', self.flow_path)
             if action == 'leave':
                 flow_id = replace_flow_action(ended_flow_id, 'focus')
@@ -932,7 +947,7 @@ class MainAppBase(ConsoleApp, ABC):
             self.call_method('on_flow_widget_focused')
             self._last_focus_flow_id = flow_id
 
-        self.vpo("CHANGED " + msg + f"+path={self.flow_path} args={event_kwargs} last_foc='{self._last_focus_flow_id}'")
+        self.vpo(f"{prefix}CHANGED path={self.flow_path} args={event_kwargs} last_foc='{self._last_focus_flow_id}'")
 
         return True
 
@@ -1530,7 +1545,7 @@ class MainAppBase(ConsoleApp, ABC):
         """ return the absolute window x and y position of the passed widget.
 
         :param wid:             widget to determine the position of.
-        :return:                tuple of x and y screen coordinate.
+        :return:                tuple of x and y screen/window coordinate.
         """
         return wid.x, wid.y
 
